@@ -13,6 +13,7 @@
 #include "monitor.h"
 #include "rules.h"
 #include "object.h"
+#include "mouse.h"
 
 #include "glib-backports.h"
 #include <string.h>
@@ -32,13 +33,16 @@ static char* completion_focus_args[]    = { "-i", "-e", NULL };
 static char* completion_unrule_flags[]   = { "-F", "--all", NULL };
 static char* completion_keyunbind_args[]= { "-F", "--all", NULL };
 static char* completion_flag_args[]     = { "on", "off", "true", "false", "toggle", NULL };
-static char* completion_userattribute_types[] = { "int", "uint", "string", "bool", NULL };
+static char* completion_userattribute_types[] = { "int", "uint", "string", "bool", "color", NULL };
 static char* completion_status[]        = { "status", NULL };
 static char* completion_special_winids[]= { "urgent", "", NULL };
 static char* completion_use_index_args[]= { "--skip-visible", NULL };
 static char* completion_cycle_all_args[]= { "--skip-invisible", NULL };
 static char* completion_pm_one[]= { "+1", "-1", NULL };
-static char* completion_split_modes[]= { "horizontal", "vertical", "left", "right", "top", "bottom", "explode", NULL };
+static char* completion_mouse_functions[]= { "move", "zoom", "resize", "call", NULL };
+static char* completion_detect_monitors_args[] =
+    { "-l", "--list", "--no-disjoin", /* TODO: "--keep-small", */ NULL };
+static char* completion_split_modes[]= { "horizontal", "vertical", "left", "right", "top", "bottom", "explode", "auto", NULL };
 static char* completion_split_ratios[]= {
     "0.1", "0.2", "0.3", "0.4", "0.5", "0.6", "0.7", "0.8", "0.9", NULL };
 
@@ -48,8 +52,10 @@ static bool no_completion(int argc, char** argv, int pos) {
 
 static bool first_parameter_is_tag(int argc, char** argv, int pos);
 static bool first_parameter_is_flag(int argc, char** argv, int pos);
+static bool second_parameter_is_call(int argc, char** argv, int pos);
 static bool first_parameter_is_writable_attribute(int argc, char** argv, int pos);
 static bool parameter_expected_offset(int argc, char** argv, int pos, int offset);
+static bool parameter_expected_offset_1(int argc, char** argv, int pos);
 static bool parameter_expected_offset_2(int argc, char** argv, int pos);
 static bool parameter_expected_offset_3(int argc, char** argv, int pos);
 
@@ -69,6 +75,9 @@ struct {
     { "reload",         1,  no_completion },
     { "true",           1,  no_completion },
     { "false",          1,  no_completion },
+    { "!",              2,  parameter_expected_offset_1 },
+    { "try",            2,  parameter_expected_offset_1 },
+    { "silent",         2,  parameter_expected_offset_1 },
     { "version",        1,  no_completion },
     { "list_commands",  1,  no_completion },
     { "list_monitors",  1,  no_completion },
@@ -78,7 +87,8 @@ struct {
     { "unlock",         1,  no_completion },
     { "keybind",        2,  parameter_expected_offset_2 },
     { "keyunbind",      2,  no_completion },
-    { "mousebind",      3,  no_completion },
+    { "mousebind",      3,  second_parameter_is_call },
+    { "mousebind",      3,  parameter_expected_offset_3 },
     { "mouseunbind",    1,  no_completion },
     { "focus_nth",      2,  no_completion },
     { "cycle",          2,  no_completion },
@@ -116,7 +126,6 @@ struct {
     { "move_index",     3,  no_completion },
     { "lock_tag",       2,  no_completion },
     { "unlock_tag",     2,  no_completion },
-    { "detect_monitors",1,  no_completion },
     { "add_monitor",    7,  no_completion },
     { "rename_monitor", 3,  no_completion },
     { "remove_monitor", 2,  no_completion },
@@ -182,6 +191,7 @@ struct {
     { "cycle_all",      EQ, 2,  .list = completion_pm_one },
     { "cycle_monitor",  EQ, 1,  .list = completion_pm_one },
     { "dump",           EQ, 1,  .function = complete_against_tags },
+    { "detect_monitors", GE, 1,  .list = completion_detect_monitors_args },
     { "floating",       EQ, 1,  .function = complete_against_tags },
     { "floating",       EQ, 1,  .list = completion_flag_args },
     { "floating",       EQ, 1,  .list = completion_status },
@@ -198,11 +208,16 @@ struct {
     { "move",           EQ, 1,  .function = complete_against_tags },
     { "move_index",     EQ, 2,  .list = completion_use_index_args },
     { "or",             GE, 1,  .function = complete_chain },
-    { "!",              GE, 1,  .function = complete_negate },
+    { "!",              GE, 1,  .function = complete_against_commands_1 },
+    { "try",            GE, 1,  .function = complete_against_commands_1 },
+    { "silent",         GE, 1,  .function = complete_against_commands_1 },
     { "pseudotile",     EQ, 1,  .list = completion_flag_args },
     { "keybind",        GE, 1,  .function = complete_against_keybind_command },
     { "keyunbind",      EQ, 1,  .list = completion_keyunbind_args },
     { "keyunbind",      EQ, 1,  .function = complete_against_keybinds },
+    { "mousebind",      EQ, 1,  .function = complete_against_mouse_combinations },
+    { "mousebind",      EQ, 2,  .list = completion_mouse_functions },
+    { "mousebind",      GE, 3,  .function = complete_against_commands_3 },
     { "rename",         EQ, 1,  .function = complete_against_tags },
     { "raise",          EQ, 1,  .list = completion_special_winids },
     { "raise",          EQ, 1,  .function = complete_against_winids },
@@ -422,17 +437,6 @@ void complete_against_tags(int argc, char** argv, int pos, GString* output) {
     }
 }
 
-void complete_negate(int argc, char** argv, int pos, GString* output) {
-    if (pos <= 0) {
-        return;
-    }
-    // Remove the ! from the argv
-    (void)SHIFT(argc, argv);
-    pos--;
-    // Complete as normal
-    complete_against_commands(argc, argv, pos, output);
-}
-
 void complete_against_monitors(int argc, char** argv, int pos, GString* output) {
     char* needle;
     if (pos >= argc) {
@@ -695,6 +699,28 @@ void complete_against_keybind_command(int argc, char** argv, int position,
     }
 }
 
+void complete_against_mouse_combinations(int argc, char** argv, int position,
+                                         GString* output)
+{
+    if (argc < 1 || position < 1) {
+        return;
+    }
+    // complete the mouse combination
+    char* needle = (position < argc) ? argv[position] : "";
+    char* lasttok = strlasttoken(needle, KEY_COMBI_SEPARATORS);
+    char* prefix = g_strdup(needle);
+    prefix[lasttok - needle] = '\0';
+    char separator = KEY_COMBI_SEPARATORS[0];
+    if (lasttok != needle) {
+        // if there is a suffix, then the already used separator is before
+        // the start of the last token
+        separator = lasttok[-1];
+    }
+    complete_against_modifiers(lasttok, separator, prefix, output);
+    complete_against_mouse_buttons(lasttok, prefix, output);
+    g_free(prefix);
+}
+
 void complete_against_env(int argc, char** argv, int position,
                           GString* output) {
     GString* curname = g_string_sized_new(30);
@@ -711,6 +737,11 @@ void complete_against_env(int argc, char** argv, int position,
     g_string_free(curname, true);
 }
 
+void complete_against_commands_1(int argc, char** argv, int position,
+                                      GString* output) {
+    complete_against_commands(argc - 1, argv + 1, position - 1, output);
+}
+
 void complete_against_commands_3(int argc, char** argv, int position,
                                       GString* output) {
     complete_against_commands(argc - 3, argv + 3, position - 3, output);
@@ -879,6 +910,14 @@ static bool first_parameter_is_flag(int argc, char** argv, int pos) {
     }
 }
 
+static bool second_parameter_is_call(int argc, char** argv, int pos) {
+    if (argc >= 3 && !strcmp(argv[2], "call")) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
 static bool first_parameter_is_writable_attribute(int argc, char** argv, int pos) {
     GString* dummy = g_string_new("");
     HSAttribute* attr = NULL;
@@ -900,6 +939,10 @@ static bool parameter_expected_offset(int argc, char** argv, int pos, int offset
     return parameter_expected(argc - offset, argv + offset, pos - offset);
 }
 
+static bool parameter_expected_offset_1(int argc, char** argv, int pos) {
+    return parameter_expected_offset(argc,argv, pos, 1);
+}
+
 static bool parameter_expected_offset_2(int argc, char** argv, int pos) {
     return parameter_expected_offset(argc,argv, pos, 2);
 }
