@@ -65,12 +65,17 @@ void complete_against_settings(int argc, char** argv, int pos, GString* output);
 void complete_against_keybinds(int argc, char** argv, int pos, GString* output);
 int complete_against_commands(int argc, char** argv, int position,
                               GString* output);
+void complete_against_commands_1(int argc, char** argv, int position,
+                                 GString* output);
 void complete_against_commands_3(int argc, char** argv, int position,
                                  GString* output);
 void complete_against_arg_1(int argc, char** argv, int position, GString* output);
 void complete_against_arg_2(int argc, char** argv, int position, GString* output);
 void complete_against_keybind_command(int argc, char** argv, int position,
                                       GString* output);
+void complete_against_mouse_combinations(int argc, char** argv, int position,
+                                      GString* output);
+
 void complete_against_env(int argc, char** argv, int position, GString* output);
 void complete_chain(int argc, char** argv, int position, GString* output);
 
