@@ -1,4 +1,4 @@
-/** Copyright 2011-2012 Thorsten Wißmann. All rights reserved.
+/** Copyright 2011-2013 Thorsten Wißmann. All rights reserved.
  *
  * This software is licensed under the "Simplified BSD License".
  * See LICENSE for details */
@@ -12,7 +12,7 @@
 #include <stdio.h>
 #include <stdarg.h>
 // other
-#include <glib.h>
+#include "glib-backports.h"
 // gui
 #include <X11/Xlib.h>
 #include <X11/Xproto.h>
