@@ -14,7 +14,9 @@
 #include "layout.h"
 #include "stack.h"
 #include "monitor.h"
+#include "floating.h"
 
+#include <glib.h>
 #include "glib-backports.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -41,6 +43,7 @@ int* g_direction_external_only;
 int* g_gapless_grid;
 int* g_smart_frame_surroundings;
 int* g_smart_window_surroundings;
+int* g_focus_crosses_monitor_boundaries;
 int* g_frame_padding;
 unsigned long g_frame_border_active_color;
 unsigned long g_frame_border_normal_color;
@@ -79,23 +82,24 @@ static void fetch_frame_colors() {
     g_gapless_grid = &(settings_find("gapless_grid")->value.i);
     g_smart_frame_surroundings = &(settings_find("smart_frame_surroundings")->value.i);
     g_smart_window_surroundings = &(settings_find("smart_window_surroundings")->value.i);
+    g_focus_crosses_monitor_boundaries = &(settings_find("focus_crosses_monitor_boundaries")->value.i);
     *g_default_frame_layout = CLAMP(*g_default_frame_layout, 0, LAYOUT_COUNT - 1);
-    char* str = settings_find("frame_border_normal_color")->value.s;
+    char* str = settings_find_string("frame_border_normal_color");
     g_frame_border_normal_color = getcolor(str);
-    str = settings_find("frame_border_active_color")->value.s;
+    str = settings_find_string("frame_border_active_color");
     g_frame_border_active_color = getcolor(str);
-    str = settings_find("frame_border_inner_color")->value.s;
+    str = settings_find_string("frame_border_inner_color");
     g_frame_border_inner_color = getcolor(str);
     // background color
-    str = settings_find("frame_bg_normal_color")->value.s;
+    str = settings_find_string("frame_bg_normal_color");
     g_frame_bg_normal_color = getcolor(str);
-    str = settings_find("frame_bg_active_color")->value.s;
+    str = settings_find_string("frame_bg_active_color");
     g_frame_bg_active_color = getcolor(str);
     g_frame_active_opacity = CLAMP(settings_find("frame_active_opacity")->value.i, 0, 100);
     g_frame_normal_opacity = CLAMP(settings_find("frame_normal_opacity")->value.i, 0, 100);
 
     // tree style
-    g_tree_style = settings_find("tree_style")->value.s;
+    g_tree_style = settings_find_string("tree_style");
     if (g_utf8_strlen(g_tree_style, -1) < 8) {
         g_warning("too few characters in setting tree_style\n");
         // ensure that it is long enough
@@ -174,7 +178,7 @@ void frame_insert_client(HSFrame* frame, struct HSClient* client) {
         if (g_cur_frame == frame
             && frame->content.clients.selection >= (count-1)) {
             frame->content.clients.selection = count - 1;
-            window_focus(client->window);
+            client_window_focus(client);
         }
     } else { /* frame->type == TYPE_FRAMES */
         HSLayout* layout = &frame->content.layout;
@@ -678,7 +682,6 @@ int frame_current_set_client_layout(int argc, char** argv, GString* output) {
 void frame_apply_client_layout_linear(HSFrame* frame, Rectangle rect, bool vertical) {
     HSClient** buf = frame->content.clients.buf;
     size_t count = frame->content.clients.count;
-    int selection = frame->content.clients.selection;
     Rectangle cur = rect;
     int last_step_y;
     int last_step_x;
@@ -704,7 +707,6 @@ void frame_apply_client_layout_linear(HSFrame* frame, Rectangle rect, bool verti
         // add the space, if count does not divide frameheight without remainder
         cur.height += (i == count-1) ? last_step_y : 0;
         cur.width += (i == count-1) ? last_step_x : 0;
-        client_setup_border(client, (g_cur_frame == frame) && (i == selection));
         client_resize_tiling(client, cur, frame);
         cur.y += step_y;
         cur.x += step_x;
@@ -847,9 +849,7 @@ void frame_apply_layout(HSFrame* frame, Rectangle rect) {
             return;
         }
 
-        if (!*g_smart_window_surroundings
-            || (frame->content.clients.count != 1
-                && frame->content.clients.layout != LAYOUT_MAX)) {
+        if (!smart_window_surroundings_active(frame)) {
             // apply window gap
             rect.x += *g_window_gap;
             rect.y += *g_window_gap;
@@ -963,8 +963,8 @@ int frame_current_set_selection(int argc, char** argv) {
         index = frame->content.clients.count - 1;
     }
     frame->content.clients.selection = index;
-    Window window = frame->content.clients.buf[index]->window;
-    window_focus(window);
+    HSClient* client = frame->content.clients.buf[index];
+    client_window_focus(client);
     return 0;
 }
 
@@ -988,8 +988,7 @@ int frame_current_cycle_selection(int argc, char** argv) {
     index += count;
     index %= count;
     frame->content.clients.selection = index;
-    Window window = frame->content.clients.buf[index]->window;
-    window_focus(window);
+    monitor_apply_layout(get_current_monitor());
     return 0;
 }
 
@@ -1198,6 +1197,7 @@ bool frame_split(HSFrame* frame, int align, int fraction) {
         // do nothing if tree would be to large
         return false;
     }
+    assert(frame->type == TYPE_CLIENTS);
     // ensure fraction is allowed
     fraction = CLAMP(fraction,
                      FRACTION_UNIT * (0.0 + FRAME_MIN_FRACTION),
@@ -1251,6 +1251,7 @@ int frame_split_command(int argc, char** argv, GString* output) {
         { "horizontal", ALIGN_HORIZONTAL,   true,   0   },
         { "left",       ALIGN_HORIZONTAL,   false,  1   },
         { "explode",    ALIGN_EXPLODE,      true,   0   },
+        { "auto",       align_auto,         true,   0   },
     };
     for (int i = 0; i < LENGTH(splitModes); i++) {
         if (splitModes[i].name[0] == argv[1][0]) {
@@ -1505,7 +1506,12 @@ int frame_focus_command(int argc, char** argv, GString* output) {
     }
     //HSFrame* frame = g_cur_frame;
     int index;
-    if (!external_only &&
+    bool neighbour_found = true;
+    if (g_cur_frame->tag->floating) {
+        enum HSDirection dir = char_to_direction(direction);
+        if (dir < 0) return HERBST_INVALID_ARGUMENT;
+        neighbour_found = floating_focus_direction(dir);
+    } else if (!external_only &&
         (index = frame_inner_neighbour_index(g_cur_frame, direction)) != -1) {
         g_cur_frame->content.clients.selection = index;
         frame_focus_recursive(g_cur_frame);
@@ -1522,10 +1528,25 @@ int frame_focus_command(int argc, char** argv, GString* output) {
             frame_focus_recursive(parent);
             monitor_apply_layout(get_current_monitor());
         } else {
+            neighbour_found = false;
+        }
+    }
+    if (!neighbour_found && !*g_focus_crosses_monitor_boundaries) {
+        g_string_append_printf(output,
+            "%s: No neighbour found\n", argv[0]);
+        return HERBST_FORBIDDEN;
+    }
+    if (!neighbour_found && *g_focus_crosses_monitor_boundaries) {
+        // find monitor in the specified direction
+        enum HSDirection dir = char_to_direction(direction);
+        if (dir < 0) return HERBST_INVALID_ARGUMENT;
+        int idx = monitor_index_in_direction(get_current_monitor(), dir);
+        if (idx < 0) {
             g_string_append_printf(output,
                 "%s: No neighbour found\n", argv[0]);
             return HERBST_FORBIDDEN;
         }
+        monitor_focus_by_index(idx);
     }
     return 0;
 }
@@ -1534,7 +1555,7 @@ int frame_move_window_command(int argc, char** argv, GString* output) {
     // usage: move left|right|up|down
     if (argc < 2) return HERBST_NEED_MORE_ARGS;
     if (!g_cur_frame) {
-        fprintf(stderr, "warning: no frame is selected\n");
+        fprintf(stderr, "error: no frame is selected\n");
         return HERBST_UNKNOWN_ERROR;
     }
     char direction = argv[1][0];
@@ -1547,6 +1568,16 @@ int frame_move_window_command(int argc, char** argv, GString* output) {
         external_only = true;
         direction = argv[2][0];
     }
+    if (!frame_focused_client(g_cur_frame)) {
+        return HERBST_FORBIDDEN;
+    }
+    if (is_client_floated(get_current_client())) {
+        // try to move the floating window
+        enum HSDirection dir = char_to_direction(direction);
+        if (dir < 0) return HERBST_INVALID_ARGUMENT;
+        bool success = floating_shift_direction(dir);
+        return success ? 0 : HERBST_FORBIDDEN;
+    }
     int index;
     if (!external_only &&
         (index = frame_inner_neighbour_index(g_cur_frame, direction)) != -1) {
@@ -1582,7 +1613,7 @@ int frame_move_window_command(int argc, char** argv, GString* output) {
             for (i = 0; i < count; i++) {
                 if (buf[i] == client) {
                     frame->content.clients.selection = i;
-                    window_focus(buf[i]->window);
+                    client_window_focus(buf[i]);
                     break;
                 }
             }
@@ -1719,9 +1750,9 @@ int frame_focus_recursive(HSFrame* frame) {
     frame_unfocus();
     if (frame->content.clients.count) {
         int selection = frame->content.clients.selection;
-        window_focus(frame->content.clients.buf[selection]->window);
+        client_window_focus(frame->content.clients.buf[selection]);
     } else {
-        window_unfocus_last();
+        client_window_unfocus_last();
     }
     return 0;
 }
@@ -1771,7 +1802,7 @@ static void frame_hide(HSFrame* frame) {
         HSClient** buf = frame->content.clients.buf;
         size_t count = frame->content.clients.count;
         for (i = 0; i < count; i++) {
-            window_hide(buf[i]->window);
+            client_set_visible(buf[i], false);
         }
     }
 }
@@ -1787,7 +1818,7 @@ static void frame_show_clients(HSFrame* frame) {
         HSClient** buf = frame->content.clients.buf;
         size_t count = frame->content.clients.count;
         for (i = 0; i < count; i++) {
-            window_show(buf[i]->window);
+            client_set_visible(buf[i], true);
         }
     }
 }
@@ -1886,11 +1917,7 @@ void frame_set_visible(HSFrame* frame, bool visible) {
     if (frame->window_visible == visible) {
         return;
     }
-    if (visible) {
-        window_show(frame->window);
-    } else {
-        window_hide(frame->window);
-    }
+    window_set_visible(frame->window, visible);
     frame->window_visible = visible;
 }
 
@@ -1936,8 +1963,11 @@ int frame_focus_edge(int argc, char** argv, GString* output) {
     // Puts the focus to the edge in the specified direction
     char* args[] = { "" };
     monitors_lock_command(LENGTH(args), args);
+    int oldval = *g_focus_crosses_monitor_boundaries;
+    *g_focus_crosses_monitor_boundaries = 0;
     while (0 == frame_focus_command(argc,argv,output))
         ;
+    *g_focus_crosses_monitor_boundaries = oldval;
     monitors_unlock_command(LENGTH(args), args);
     return 0;
 }
@@ -1946,9 +1976,18 @@ int frame_move_window_edge(int argc, char** argv, GString* output) {
     // Moves a window to the edge in the specified direction
     char* args[] = { "" };
     monitors_lock_command(LENGTH(args), args);
+    int oldval = *g_focus_crosses_monitor_boundaries;
+    *g_focus_crosses_monitor_boundaries = 0;
     while (0 == frame_move_window_command(argc,argv,output))
         ;
+    *g_focus_crosses_monitor_boundaries = oldval;
     monitors_unlock_command(LENGTH(args), args);
     return 0;
 }
 
+bool smart_window_surroundings_active(HSFrame* frame) {
+    return *g_smart_window_surroundings
+            && (frame->content.clients.count == 1
+                || frame->content.clients.layout == LAYOUT_MAX);
+}
+
