@@ -7,11 +7,13 @@
 #define __HERBSTLUFT_LAYOUT_H_
 
 #include "glib-backports.h"
+#include "utils.h"
 #include <stdbool.h>
 #include <stdlib.h>
 #include <X11/Xlib.h>
 #include <monitor.h>
-#include <tag.h>
+#include "tag.h"
+#include "floating.h"
 
 #define LAYOUT_DUMP_BRACKETS "()" /* must consist of exactly two chars */
 #define LAYOUT_DUMP_WHITESPACES " \t\n" /* must be at least one char */
@@ -183,5 +185,7 @@ void frame_update_border(Window window, unsigned long color);
 int frame_focus_edge(int argc, char** argv, GString* output);
 int frame_move_window_edge(int argc, char** argv, GString* output);
 
+bool smart_window_surroundings_active(HSFrame* frame);
+
 #endif
 
