@@ -40,6 +40,7 @@ typedef struct RectList {
 } RectList;
 
 static RectList* reclist_insert_disjoint(RectList* head, RectList* mt);
+static RectList* disjoin_rects(Rectangle* buf, size_t count);
 
 void monitor_init() {
     g_monitors_locked = &(settings_find("monitors_locked")->value.i);
@@ -50,6 +51,11 @@ void monitor_init() {
     g_mouse_recenter_gap       = &(settings_find("mouse_recenter_gap")->value.i);
     g_monitor_stack = stack_create();
     g_monitor_object = hsobject_create_and_link(hsobject_root(), "monitors");
+    HSAttribute attributes[] = {
+        ATTRIBUTE_UINT("count", g_monitors->len, ATTR_READ_ONLY),
+        ATTRIBUTE_LAST,
+    };
+    hsobject_set_attributes(g_monitor_object, attributes);
     g_monitor_by_name_object = hsobject_create_and_link(g_monitor_object, "by-name");
 }
 
@@ -252,6 +258,15 @@ static void rectlist_free(RectList* head) {
     rectlist_free(next);
 }
 
+static int rectlist_length_acc(RectList* head, int acc) {
+    if (!head) return acc;
+    else return rectlist_length_acc(head->next, acc + 1);
+}
+
+static int rectlist_length(RectList* head) {
+    return rectlist_length_acc(head, 0);
+}
+
 static RectList* disjoin_rects(Rectangle* buf, size_t count) {
     RectList* cur;
     struct RectList* rects = NULL;
@@ -329,6 +344,7 @@ int set_monitor_rects(Rectangle* templates, size_t count) {
     while (i < g_monitors->len) {
         remove_monitor(i);
     }
+    monitor_update_focus_objects();
     all_monitors_apply_layout();
     return 0;
 }
@@ -357,12 +373,20 @@ int string_to_monitor_index(char* string) {
     if (string[0] == '\0') {
         return g_cur_monitor;
     } else if (string[0] == '-' || string[0] == '+') {
-        // relative monitor index
-        int idx = g_cur_monitor + atoi(string);
-        idx %= g_monitors->len;
-        idx += g_monitors->len;
-        idx %= g_monitors->len;
-        return idx;
+        if (isdigit(string[1])) {
+            // relative monitor index
+            int idx = g_cur_monitor + atoi(string);
+            idx %= g_monitors->len;
+            idx += g_monitors->len;
+            idx %= g_monitors->len;
+            return idx;
+        } else if (string[0] == '-') {
+            enum HSDirection dir = char_to_direction(string[1]);
+            if (dir < 0) return -1;
+            return monitor_index_in_direction(get_current_monitor(), dir);
+        } else {
+            return -1;
+        }
     } else if (isdigit(string[0])) {
         // absolute monitor index
         int idx = atoi(string);
@@ -376,6 +400,21 @@ int string_to_monitor_index(char* string) {
     }
 }
 
+int monitor_index_in_direction(HSMonitor* m, enum HSDirection dir) {
+    int cnt = monitor_count();
+    RectangleIdx* rects = g_new0(RectangleIdx, cnt);
+    int relidx = -1;
+    FOR (i,0,cnt) {
+        rects[i].idx = i;
+        rects[i].r = monitor_with_index(i)->rect;
+        if (monitor_with_index(i) == m) relidx = i;
+    }
+    HSAssert(relidx >= 0);
+    int result = find_rectangle_in_direction(rects, cnt, relidx, dir);
+    g_free(rects);
+    return result;
+}
+
 HSMonitor* string_to_monitor(char* string) {
     int idx = string_to_monitor_index(string);
     return monitor_with_index(idx);
@@ -520,6 +559,7 @@ int remove_monitor_command(int argc, char** argv, GString* output) {
         g_string_append_printf(output,
             "%s: Can't remove the last monitor\n", argv[0]);
     }
+    monitor_update_focus_objects();
     return ret;
 }
 
@@ -791,6 +831,10 @@ int monitor_set_tag(HSMonitor* monitor, HSTag* tag) {
             ewmh_update_current_desktop();
             emit_tag_changed(other->tag, monitor_index_of(other));
             emit_tag_changed(tag, g_cur_monitor);
+        } else {
+            // if we are not allowed to steal the tag, then just focus the
+            // other monitor
+            monitor_focus_by_index(monitor_index_of(other));
         }
         return 0;
     }
@@ -895,7 +939,7 @@ int monitor_focus_command(int argc, char** argv, GString* output) {
         return HERBST_NEED_MORE_ARGS;
     }
     int new_selection = string_to_monitor_index(argv[1]);
-    if (new_selection == -1) {
+    if (new_selection < 0) {
         g_string_append_printf(output,
             "%s: Monitor \"%s\" not found!\n", argv[0], argv[1]);
         return HERBST_INVALID_ARGUMENT;
@@ -1162,19 +1206,38 @@ bool detect_monitors_xinerama(Rectangle** ret_rects, size_t* ret_count) {
 
 // monitor detection that always works: one monitor across the entire screen
 bool detect_monitors_simple(Rectangle** ret_rects, size_t* ret_count) {
+    XWindowAttributes attributes;
+    XGetWindowAttributes(g_display, g_root, &attributes);
+
     *ret_count = 1;
     *ret_rects = g_new0(Rectangle, 1);
     (*ret_rects)->x = 0;
     (*ret_rects)->y = 0;
-    (*ret_rects)->width = g_screen_width;
-    (*ret_rects)->height = g_screen_height;
+    (*ret_rects)->width = attributes.width;
+    (*ret_rects)->height = attributes.height;
     return true;
 }
 
+bool detect_monitors_debug_example(Rectangle** ret_rects, size_t* ret_count) {
+    *ret_count = 2;
+    *ret_rects = g_new0(Rectangle, 2);
+    (*ret_rects)[0].x = 0;
+    (*ret_rects)[0].y = 0;
+    (*ret_rects)[0].width = g_screen_width * 2 / 3;
+    (*ret_rects)[0].height = g_screen_height * 2 / 3;
+    (*ret_rects)[1].x = g_screen_width / 3;
+    (*ret_rects)[1].y = g_screen_height / 3;
+    (*ret_rects)[1].width = g_screen_width * 2 / 3;
+    (*ret_rects)[1].height = g_screen_height * 2 / 3;
+    return true;
+}
+
+
 int detect_monitors_command(int argc, char **argv, GString* output) {
     MonitorDetection detect[] = {
         detect_monitors_xinerama,
         detect_monitors_simple,
+        detect_monitors_debug_example, // move up for debugging
     };
     Rectangle* monitors = NULL;
     size_t count = 0;
@@ -1186,24 +1249,59 @@ int detect_monitors_command(int argc, char **argv, GString* output) {
         }
     }
     assert(count && monitors);
-
-    // apply it
-    int ret = set_monitor_rects(monitors, count);
-    g_free(monitors);
-    if (ret == HERBST_TAG_IN_USE && output != NULL) {
-        g_string_append_printf(output,
-            "%s: There are not enough free tags\n", argv[0]);
+    bool list_only = false;
+    bool disjoin = true;
+    //bool drop_small = true;
+    FOR (i,1,argc) {
+        if      (!strcmp(argv[i], "-l"))            list_only = true;
+        else if (!strcmp(argv[i], "--list"))        list_only = true;
+        else if (!strcmp(argv[i], "--no-disjoin"))  disjoin = false;
+        // TOOD:
+        // else if (!strcmp(argv[i], "--keep-small"))  drop_small = false;
+        else {
+            g_string_append_printf(output,
+                "detect_monitors: unknown flag \"%s\"\n", argv[i]);
+            return HERBST_INVALID_ARGUMENT;
+        }
     }
+
+    int ret = 0;
+    if (list_only) {
+        FOR (i,0,count) {
+            g_string_append_printf(output, "%dx%d%+d%+d\n",
+                monitors[i].width, monitors[i].height,
+                monitors[i].x, monitors[i].y);
+        }
+    } else {
+        // possibly disjoin them
+        if (disjoin) {
+            RectList* rl = disjoin_rects(monitors, count);
+            count = rectlist_length(rl);
+            monitors = g_renew(Rectangle, monitors, count);
+            RectList* cur = rl;
+            FOR (i,0,count) {
+                monitors[i] = cur->rect;
+                cur = cur->next;
+            }
+        }
+        // apply it
+        ret = set_monitor_rects(monitors, count);
+        if (ret == HERBST_TAG_IN_USE && output != NULL) {
+            g_string_append_printf(output,
+                "%s: There are not enough free tags\n", argv[0]);
+        }
+    }
+    g_free(monitors);
     return ret;
 }
 
-int monitor_stack_window_count(bool only_clients) {
-    return stack_window_count(g_monitor_stack, only_clients);
+int monitor_stack_window_count(bool real_clients) {
+    return stack_window_count(g_monitor_stack, real_clients);
 }
 
-void monitor_stack_to_window_buf(Window* buf, int len, bool only_clients,
+void monitor_stack_to_window_buf(Window* buf, int len, bool real_clients,
                                  int* remain_len) {
-    stack_to_window_buf(g_monitor_stack, buf, len, only_clients, remain_len);
+    stack_to_window_buf(g_monitor_stack, buf, len, real_clients, remain_len);
 }
 
 HSStack* get_monitor_stack() {
@@ -1236,8 +1334,8 @@ void monitor_restack(HSMonitor* monitor) {
     /* remove a focused fullscreen client */
     HSClient* client = frame_focused_client(monitor->tag->frame);
     if (client && client->fullscreen) {
-        XRaiseWindow(g_display, client->window);
-        int idx = array_find(buf, count, sizeof(*buf), &client->window);
+        XRaiseWindow(g_display, client->dec.decwin);
+        int idx = array_find(buf, count, sizeof(*buf), &client->dec.decwin);
         assert(idx >= 0);
         count--;
         memmove(buf + idx, buf + idx + 1, sizeof(*buf) * (count - idx));
@@ -1276,3 +1374,13 @@ void drop_enternotify_events() {
     XSync(g_display, False);
     while(XCheckMaskEvent(g_display, EnterWindowMask, &ev));
 }
+
+Rectangle monitor_get_floating_area(HSMonitor* m) {
+    Rectangle r = m->rect;
+    r.x += m->pad_left;
+    r.width -= m->pad_left + m->pad_right;
+    r.y += m->pad_up;
+    r.height -= m->pad_up + m->pad_down;
+    return r;
+}
+
