@@ -13,6 +13,7 @@
 #include <X11/extensions/Xinerama.h>
 #endif /* XINERAMA */
 
+#include "floating.h"
 #include "object.h"
 #include "utils.h"
 
@@ -56,6 +57,7 @@ HSMonitor* monitor_with_coordinate(int x, int y);
 HSMonitor* monitor_with_index(int index);
 HSMonitor* find_monitor_with_tag(struct HSTag* tag);
 HSMonitor* add_monitor(Rectangle rect, struct HSTag* tag, char* name);
+Rectangle monitor_get_floating_area(HSMonitor* m);
 void monitor_focus_by_index(int new_selection);
 int monitor_get_relative_x(HSMonitor* m, int x_root);
 int monitor_get_relative_y(HSMonitor* m, int y_root);
@@ -66,6 +68,7 @@ int find_monitor_index_by_name(char* name);
 HSMonitor* find_monitor_by_name(char* name);
 HSMonitor* string_to_monitor(char* string);
 int string_to_monitor_index(char* string);
+int monitor_index_in_direction(HSMonitor* m, enum HSDirection dir);
 int add_monitor_command(int argc, char** argv, GString* output);
 int monitor_raise_command(int argc, char** argv, GString* output);
 int remove_monitor_command(int argc, char** argv, GString* output);
@@ -100,8 +103,8 @@ void all_monitors_replace_previous_tag(struct HSTag* old, struct HSTag* new);
 void drop_enternotify_events();
 
 void monitor_restack(HSMonitor* monitor);
-int monitor_stack_window_count(bool only_clients);
-void monitor_stack_to_window_buf(Window* buf, int len, bool only_clients,
+int monitor_stack_window_count(bool real_clients);
+void monitor_stack_to_window_buf(Window* buf, int len, bool real_clients,
                                  int* remain_len);
 struct HSStack* get_monitor_stack();
 
