@@ -63,6 +63,8 @@ DECLARE_CONSEQUENCE(consequence_switchtag);
 DECLARE_CONSEQUENCE(consequence_ewmhrequests);
 DECLARE_CONSEQUENCE(consequence_ewmhnotify);
 DECLARE_CONSEQUENCE(consequence_hook);
+DECLARE_CONSEQUENCE(consequence_keymask);
+DECLARE_CONSEQUENCE(consequence_monitor);
 
 /// GLOBALS ///
 
@@ -91,6 +93,8 @@ static HSConsequenceType g_consequence_types[] = {
     { "ewmhrequests",   consequence_ewmhrequests    },
     { "ewmhnotify",     consequence_ewmhnotify      },
     { "hook",           consequence_hook            },
+    { "keymask",        consequence_keymask         },
+    { "monitor",        consequence_monitor         },
 };
 
 static GQueue g_rules = G_QUEUE_INIT; // a list of HSRule* elements
@@ -559,6 +563,7 @@ void client_changes_init(HSClientChanges* changes, HSClient* client) {
     changes->switchtag = false;
     changes->manage = true;
     changes->fullscreen = ewmh_is_fullscreen_set(client->window);
+    changes->keymask = g_string_new("");
 }
 
 void client_changes_free_members(HSClientChanges* changes) {
@@ -569,6 +574,9 @@ void client_changes_free_members(HSClientChanges* changes) {
     if (changes->tree_index) {
         g_string_free(changes->tree_index, true);
     }
+    if (changes->monitor_name) {
+        g_string_free(changes->monitor_name, true);
+    }
 }
 
 // apply all rules to a certain client an save changes
@@ -821,3 +829,20 @@ void consequence_hook(HSConsequence* cons, HSClient* client,
     g_string_free(winid, true);
 }
 
+void consequence_keymask(HSConsequence* cons,
+                         HSClient* client, HSClientChanges* changes) {
+    if (changes->keymask) {
+        g_string_assign(changes->keymask, cons->value.str);
+    } else {
+        changes->keymask = g_string_new(cons->value.str);
+    }
+}
+
+void consequence_monitor(HSConsequence* cons, HSClient* client,
+                            HSClientChanges* changes) {
+    if (changes->monitor_name) {
+        g_string_assign(changes->monitor_name, cons->value.str);
+    } else {
+        changes->monitor_name = g_string_new(cons->value.str);
+    }
+}
