@@ -69,8 +69,8 @@ void stack_clear_layer(HSStack* stack, HSLayer layer);
 int print_stack_command(int argc, char** argv, GString* output);
 
 // returns the number of windows in this stack
-int stack_window_count(HSStack* stack, bool only_clients);
-void stack_to_window_buf(HSStack* stack, Window* buf, int len, bool only_clients,
+int stack_window_count(HSStack* stack, bool real_clients);
+void stack_to_window_buf(HSStack* stack, Window* buf, int len, bool real_clients,
                          int* remain_len);
 void stack_restack(HSStack* stack);
 Window stack_lowest_window(HSStack* stack);
