@@ -56,13 +56,22 @@ void die(const char *errstr, ...) {
 // get X11 color from color string
 // from dwm.c
 unsigned long getcolor(const char *colstr) {
+    HSColor ret_color;
+    if (!getcolor_error(colstr, &ret_color)) {
+        ret_color = 0;
+    }
+    return ret_color;
+}
+
+bool getcolor_error(const char *colstr, HSColor* ret_color) {
     Colormap cmap = DefaultColormap(g_display, g_screen);
     XColor color;
     if(!XAllocNamedColor(g_display, cmap, colstr, &color, &color)) {
         g_warning("error, cannot allocate color '%s'\n", colstr);
-        return 0;
+        return false;
     }
-    return color.pixel;
+    *ret_color = color.pixel;
+    return true;
 }
 
 // inspired by dwm's gettextprop()
@@ -155,6 +164,7 @@ bool window_has_property(Display* dpy, Window window, char* prop_name) {
 
 // duplicates an argument-vector
 char** argv_duplicate(int argc, char** argv) {
+    if (argc <= 0) return NULL;
     char** new_argv = malloc(sizeof(char*) * argc);
     if (!new_argv) {
         die("cannot malloc - there is no memory available\n");
@@ -168,6 +178,7 @@ char** argv_duplicate(int argc, char** argv) {
 
 // frees all entries in argument-vector and then the vector itself
 void argv_free(int argc, char** argv) {
+    if (argc <= 0) return;
     int i;
     for (i = 0; i < argc; i++) {
         free(argv[i]);
@@ -198,11 +209,11 @@ char* strlasttoken(char* str, char* delim) {
 }
 
 
-bool string_to_bool(char* string, bool oldvalue) {
+bool string_to_bool(const char* string, bool oldvalue) {
     return string_to_bool_error(string, oldvalue, NULL);
 }
 
-bool string_to_bool_error(char* string, bool oldvalue, bool* error) {
+bool string_to_bool_error(const char* string, bool oldvalue, bool* error) {
     bool val = oldvalue;
     if (error) {
         error = false;
